# Generated by roxygen2: do not edit by hand

S3method(plot,FFTrees)
S3method(predict,FFTrees)
S3method(print,FFTrees)
S3method(summary,FFTrees)
export(FFTrees)
export(FFTrees.guide)
export(add_fft_df)
export(add_nodes)
export(describe_data)
export(drop_nodes)
export(edit_nodes)
export(fftrees_apply)
export(fftrees_create)
export(fftrees_cuerank)
export(fftrees_define)
export(fftrees_ffttowords)
export(fftrees_threshold_factor_grid)
export(fftrees_threshold_numeric_grid)
export(fftrees_wordstofftrees)
export(flip_exits)
export(get_best_tree)
export(get_exit_type)
export(get_fft_df)
export(inwords)
export(read_fft_df)
export(reorder_nodes)
export(select_nodes)
export(showcues)
export(write_fft_df)
import(testthat)
importFrom(caret,confusionMatrix)
importFrom(cli,cli_progress_bar)
importFrom(cli,cli_progress_done)
importFrom(cli,cli_progress_update)
importFrom(cli,style_underline)
importFrom(dplyr,"%>%")
importFrom(dplyr,near)
importFrom(dplyr,pull)
importFrom(grDevices,col2rgb)
importFrom(grDevices,gray)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(knitr,kable)
importFrom(magrittr,"%>%")
importFrom(scales,comma)
importFrom(scales,percent)
importFrom(stats,as.formula)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(testthat,expect_true)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,browseVignettes)
importFrom(utils,vignette)
